function [ w,x,werror ] = ALL_productDifference( vpa_flag, points, mom_qmom)

% ALL_PRODUCTDIFFERENCE uses a matrix inversion procedure to calculate the 
% quadrature points of the distribution from its lower-order moments.
% Product difference is described fully in Marchisio et al. (2003)
%
%   INPUTS: 
%   vpa_flag       - precision flag
%                    0 -> use default precision
%                    1 -> use variable-precision floating-point arithmetic
%   points         - Number of quadrature points
%   mom_qmom       - Moment set
%
%   OUTPUTS:
%   w              - weights associated with the moment set
%   x              - abscissas associated with the moment set
%   werror         - error flag

    % Toggle warning messages on/off (they can impede runtimes)
    warning('off','MATLAB:nearlySingularMatrix')
    
    % Initialize error flag
    werror = 0;
    
    % Number of rows/columns in the triangular array of elements P
    % [Eq. 20, Marchisio and Fox (2003)]
    P_max = 2*points + 1; 
    
    % Initialize vectors
    alpha = zeros(1,2*points);  % alpha is used to find a and b
    a = zeros(1,points);        % diagonal of the Jacobi matrix
    b = zeros(1, points-1);     % co-diagonal of the Jacobi matrix
    x = zeros(1, points);       % abscissas
    w = zeros(1,points);        % weights
    
    % Initialize the Jacobi matrix
    J = zeros(points, points);
    
    % Set up P:
    % First column
    if(vpa_flag == 0)
        P(1,1) = 1; 
    else
        P(1,1) = vpa(1); 
    end
    
    % Second row
    for i = 1:(P_max-1)
        if(vpa_flag == 0)
            P(i,2) = ((-1)^(i-1))*mom_qmom(i);
        else
            P(i,2) = ((-1)^(i-1))*vpa(mom_qmom(i));
        end
    end
    
    % Fill out the rest of P [Eq. 20, Marchisio and Fox (2003)]
    for j = 3:P_max   
        for i = 1:(P_max-j+1)
            if(vpa_flag == 0)
                P(i,j) = (P(1,j-1)*P(i+1,j-2)-P(1,j-2)*P(i+1,j-1));   
            else
                P(i,j) = (vpa(P(1,j-1))*vpa(P(i+1,j-2))-vpa(P(1,j-2))*vpa(P(i+1,j-1))); 
            end
        end
    end
    
    % Calculate alpha [Eq. 21, Marchisio and Fox (2003)]
    for ind=2:2*points
        alpha(1,ind) = (P(1,ind+1)/(P(1,ind)*P(1,ind-1)));
    end
    
    % Find diagonal and co-diagonal of the Jacobi matrix
    for n = 1:points
        %[Eq. 22, Marchisio and Fox (2003)]
        a(1,n) = alpha(2*n)+alpha(2*n-1);
        
        %[Eq. 23, Marchisio and Fox (2003)]
        if(n <= (points - 1))
            b(1,n) = -(sqrt(alpha(2*n+1)*alpha(2*n)));
        end
    end
    
    % Fill Jacobi matrix
    for i2 = 1:points
        for j2 = 1:points

          if(i2==j2)
              J(i2,j2) = a(1,i2);
              if(i2 <= (points-1))
                J(i2,j2+1) = b(1,i2);
                J(i2+1,j2) = b(1,i2);
              end
          end
          
        end
    end

    % Check for NaN or Inf in the matrix and toggle error flag if found
    if(sum(sum(isnan(J))) == 0 && sum(sum(isinf(J))) == 0)
        [V,D] = eig(J); % Find the eigenvalues and eigenvectors of the Jacobi matrix
        V = V.^2;
    else
        werror = 1;
        return
    end

    for n = 1:points
      x(1,n) = D(n,n); % Abscissas are the eigenvalues
      
      % Calculate the weights from the eigenvectors 
      % [Eq. 24, Marchisio and Fox (2003)]
      w(1,n) = V(1,n)*mom_qmom(1);  
    end

    return
end

% Marchisio, D. L., J. T. Pikturna, R. O. Fox, R. D. Vigil, and A. A. 
% Barresi (2003), Quadrature method of moments for population?balance 
% equations, AIChE Journal, 49(5), 1266-1276.
