function [stats] = ANALYT(case_id, sink, num_moments, tvec, N0, S, ...
    rho_p, k_S_pb, pow, prefac_agg, Df, ln_params, char_rad0, min_m, max_m)

% ANALYT find the analytical solution describing changes in the statistics 
% of an initially lognormally distributed nanoparticle population over time
% during dissolution (under sink conditions) or aggregation
%    
%   INPUTS:
%   case_id        - Test case flag:   
%                    1 -> Dissolution only
%                    2 -> Aggregation only 
%                    3 -> Simultaneous dissolution and aggregation
%   sink           - Sink flag:        
%                    0 -> Dissolution to equilibrium ion concentration
%                    1 -> Dissolution under sink conditions (no ion)
%
%   num_moments    - Number of raw fractional moments for which to solve
%   tvec           - Vector of the times in days at which to solve the
%                    statistics of interest for the current run
%
%   N0             - Initial particle number concentration in #/m^3
%   S              - Equilibrium ion concentration (solubility) in g/m^3
%   rho_p          - NP density in g/m^3
%   k_S_pb         - Dissolution rate
%   pow            - Power that the size coordinate is raised to in the
%                    rate law for dissolution
%   prefac_agg     - Prefactor for the aggregation rate
%   Df             - Assumed fractal dimension of aggregates
%   ln_params      - Vector containing the parameters of the initial
%                    lognormal nanoparticle size distribution: [mu, sigma]
%   char_rad0      - Initial characterstic radius in m
%   min_m          - lower bound of the size domain in m
%   max_m          - upper bound of the size domain in m
%
%   OUTPUTS:
%   stats          - Matrix of statistics of the distribution as a function
%                    of time (calculated from the raw fractional moments) 
%                    to be returned to CONTROL_FILE.m for visualization and 
%                    error analysis

    % Number of time steps in the simulation
    n_t = length(tvec);

    % Initially holds the raw fractional moments but is later converted
    % to the desired statistics of the distribution (calculated from the 
    % raw moments) that are returned by the function
    stats = zeros(num_moments, n_t);

    
    % CASE 1: "Dissolution only" under sink conditions
    if(case_id == 1 && sink == 1) 
        
        % Holds the characteristic (surface-weighted geometric mean) radius
        char_rad = zeros(1,n_t); 

        % For every time step ...
        for t_index=1:n_t

           % Grab the time in days for the current time step
           t = tvec(t_index);
           
           disp(t);
           
           % Analytical solution for the pdf is from Williams et al (1982)
           % as described by Dale et al. (2016) [Eq. S49]
           % The expression for the pdf is Eq. S46
           m_t = @(m,t) (m.^(1-pow)-(1-pow)*(-k_S_pb*S)*t).^(1/(1-pow)); 
           pdf = @(m) (1 -((1-pow)*(-k_S_pb*S)*t)./(m.^(1-pow))).^(pow/(1-pow)).*N0.*((1./(ln_params(2)*sqrt(2*pi).*( m_t(m,t) ))).*exp(-(( log(m_t(m,t))-ln_params(1)).^2) ./ (2*(ln_params(2)^2)) ));
            
           % Calculate the raw fractional moments of the distribution
           for j_ind = 1:num_moments
               fun = @(m) (m.^((j_ind-1)/3)).* (pdf(m));
               stats(j_ind,t_index) =   quadgk(fun, min_m, max_m);
           end
           
           % Calculate the characteristic radius in m according to 
           % Eq. S3 of Dale et al. (2016)
           % (expressed in terms of particle mass instead of diameter 
           % by assuming spherical particles)
           char_rad_fun = @(m) ((m.^(2/3)).*log((((6/(rho_p*pi))^(1/3)))*m.^(1/3))).*pdf(m);
           char_rad(1,t_index) =   exp(quadgk(char_rad_fun, min_m, max_m)./stats(3,t_index))/2;
        end

        % Convert the characteristic radius in m to a characteristic 
        % diameter in nm
        stats(2,:) = 2*(10^9)*char_rad(1,:);  
        
        % Use the geometry of a sphere to convert the raw 2/3 moment to 
        % the surface area concentration
        stats(3,:) = stats(3,:)*(4*pi)*((4/3)*rho_p*pi)^(-(2/3)); 
   

    % CASE 2: Aggregation only
    elseif(case_id == 2) 

        % Characteristic particle mass in g 
        % (from the geometry of a sphere)
        char_mass0 = rho_p*(4/3)*pi*(char_rad0^3);

        % Characteristic particle surface area in m^2 
        % (from the geometry of a sphere)
        char_sa0 = 4*pi*(char_rad0^2); 
        
        % Holds the the geometric mean number of primary particles 
        % in the aggregate
        mean_num_per = zeros(1,n_t); 
        
        % Holds the standard deviation of the lognormal distribution 
        % at time t
        sigln_vec = zeros(1, n_t); 
        
        % Holds the mean of the lognormal distribution at time t
        vdln_vec = zeros(1, n_t); 
        
        % Holds the number concentration at time t
        N_vec = zeros(1, n_t);
        
        % Calculate the initial mean and standard deviation from ln_params
        % (i.e., convert the distributions parameters used elsewhere in our
        % work to those used by Park et al. (2000))
        vdln0 = exp(ln_params(1));
        sigln0 = exp(ln_params(2)/3);

        % For every time step ...
        for t_index=1:n_t

            % Grab the time in days for the current time step
            t = tvec(t_index);
            
            disp(t);
            
            % Analytical solution for the pdf from Park et al (2000), as 
            % described by Dale et al. (2016) [Eq. S50]
            % The pdf is given as [Eq. S46]
            X = exp(9*(log(sigln0))^2)-2;
            Y = 1+(1+exp((9*(log(sigln0))^2)/(Df^2)))*prefac_agg*N0*t ;
            sigln_vec(1,t_index) = exp(sqrt((1/9)*log(2 + (X/Y))));
            vdln_vec(1,t_index) = vdln0*(Y*exp((9*(log(sigln0))^2)/2))/sqrt(2 + (X/Y));
            N_vec(1, t_index) = N0/Y;

            pdf = @(m) N_vec(1, t_index).*((1./(    (3*log(sigln_vec(1,t_index)))     *sqrt(2*pi).*m)).*exp(-     (( log(m)-     log(vdln_vec(1,t_index))    ).^2)  ./ (2*(    (3*log(sigln_vec(1,t_index)))   ^2))       ))       ;
            
            % Calculate the raw fractional moments of the distribution
            for j_ind = 1:num_moments
                fun = @(m) max(0,(m.^((j_ind-1)/3)).*pdf(m));
                stats(j_ind,t_index) =   integral(fun, min_m*1.0001, max_m*0.999);
            end
             
            % Calculate the geometric mean number of primary particles in 
            % the aggregate
            mean_num_per_fun = @(m) log(m/char_mass0).*pdf(m);
            mean_num_per(1,t_index) =   exp( quadgk(mean_num_per_fun, min_m*1.0001, max_m*0.999) ./stats(1,t_index) );

        end

        % Characteristic diameter of the aggregate as described by Eq. S10
        % of Dale et al. (2016)
        stats(2,:)=char_rad0*((  mean_num_per(1,:)  ).^(1/Df))*2*(10^9);  
        
        % The surface area concentration does not change over time in the 
        % case of aggregation without dissolution or particle fusion
        stats(3,:)=N0*char_sa0; 
     
    end
    
    return
end


% KEY REFERENCES
% For a complete list of references, see Dale et al. (2016)

% Park, S., R. Xiang, and K. Lee (2000), Brownian coagulation of fractal 
% agglomerates: Analytical solution using the log-normal size distribution 
% assumption, Journal of colloid and interface science, 231(1), 129-135.

% Williams, M. (1983), The time-dependent behavior of aerosols with growth 
% and deposition. I. Without coagulation, Journal of Colloid and Interface 
% Science, 93(1), 252-263.
