function [ x_ab, w_ab ] = BEQMOM_2ndQuadrature( points, points2, sigma, x)

% BEQMOM_2ndQuadrature uses a matrix inversion procedure similar to 
% Product Difference to calculate the secondary quadrature points of the 
% distribution from its lower-order moments.  Details can be found in Shen
% et al. (2011), chapter section 3.2 (see in particular "3.2.3. 
% Computation of Nodes and Weights")
%
%   INPUTS:
%   points         - Number of primary quadrature points   
%   points2        - Number of secondary quadrature points 
%   sigma          - Spread of beta distrbution around each primary
%                    quadrature point
%   x              - Vector of abscissas for the distribution at the given
%                    time step
%
%   OUTPUTS:
%   x_ab           - Matrix of secondary abscissas 
%   w_ab           - Matrix of secondary weights

    % Toggle warning messages on/off (they can impede runtimes)
    warning('off','MATLAB:nearlySingularMatrix')
    
    % Initialize matrices to hold secondary abscissas and weights
    x_ab = zeros(points,points2);
    w_ab = zeros(points,points2);
    
    
    for ind2 = 1:points

        % Calculate the parameters of the weight function from the
        % parameters of the beta distribution
        alpha_ab = ((1-x(ind2))/sigma)-1;
        beta_ab = (x(ind2)/sigma)-1;
        
        % Initialize elements of the tridiagonal matrix
        % [Eq. 3.142, Shen et al. (2011)]
        a_ab = zeros(points2);
        b_ab = zeros(points2);
        
        % Use the recurrence relations 3.143a-b [Shen et al. (2011)] to
        % find the elements of the tridiagonal matrix J_ab and fill it
        for ind = 1:points2
            a_ab(ind) = (beta_ab^2-alpha_ab^2)/((2*(ind-1)+alpha_ab+beta_ab)*(2*(ind-1)+alpha_ab+beta_ab+2));
            if(ind > 1)
                b_ab(ind) = (4*(ind-1)*(ind-1+alpha_ab)*(ind-1+beta_ab)*(ind-1+alpha_ab+beta_ab))/...
                            ((2*(ind-1)+alpha_ab+beta_ab-1)*((2*(ind-1)+alpha_ab+beta_ab)^2)*(2*(ind-1)+alpha_ab+beta_ab+1));
            end
        end

        J_ab = zeros(points2, points2);
        for i2 = 1:points2
            for j2 = 1:points2

              if(i2==j2)
                  J_ab(i2,j2)=a_ab(i2);
                  if(i2 <= (points2-1))
                    J_ab(i2,j2+1)=sqrt(b_ab(i2+1));
                    J_ab(i2+1,j2)=sqrt(b_ab(i2+1));
                  end
              end

            end
        end

        % As in Product Difference, the abscissas are the eigenvalues
        % and the weights are calculated from the eigenvectors
        [V_ab,D_ab] = eig(J_ab);
        V_ab = V_ab.^2;

        for n = 1:points2
              x_ab(ind2,n) = D_ab(n,n);
              w_ab(ind2,n) = V_ab(1,n);
        end

    end

end

% Shen, J., T. Tang, and L.-L. Wang (2011), Spectral methods: algorithms, 
% analysis and applications, Springer Science & Business Media