function [ J, w, x, use_prev_flag ] = BEQMOM_calcJ(points, sigma, m_2Nminus, m_2n)

% As part of the search for sigma_b and the primary quadrature points in 
% BEQMOM, BEQMOM_CALCJ calculates J(sigma_b) (Dale et al. 2016, Eq. S39) 
%
%   INPUTS: 
%   points         - Number of primary quadrature points 
%   sigma          - Current guess for sigma_b
%   m_2Nminus      - (transformed) moment set (k=0 to 2N-1)
%   m_2n           - (transformed) 2Nth moment (k=2N)
%
%   OUTPUTS:
%   J              - J(sigma_b), the difference between m_2n and the value 
%                    of the 2Nth moment suggested by the current guess for 
%                    sigma_b
%   w              - weights associated with the current guess for sigma_b
%   x              - abscissas associated with the current guess for
%                    sigma_b
%   use_prev_flag  - error flag

    % Toggle warning messages on/off (they can impede runtimes)
    warning('off','MATLAB:nearlySingularMatrix')
    
    % Create coefficient matrix for the right hand side of the system of 
    % equations suggested by Eq. S38 for k = 0 to 2*N-1
    % Note that, in this simple implementation, 'points' cannot exceed 3
    A = zeros(2*points,2*points);

    for i = 1:2*points
        for j = 1:2*points
            if(i==j) 
                A(i,j) = 1; 
            end

            if(points > 1)
                A(3,3) = (1/(1+sigma));
                A(3,2) = (1/(1+sigma))*sigma^1*1;
            
                A(4,4) = (1/((1+2*sigma)*(1+sigma)));
                A(4,3) = (1/((1+2*sigma)*(1+sigma)))*sigma^1*3;
                A(4,2) = (1/((1+2*sigma)*(1+sigma)))*sigma^2*2;
            end

            if(points > 2)
                A(5,5) = (1/((1+3*sigma)*(1+2*sigma)*(1+sigma)));
                A(5,4) = (1/((1+3*sigma)*(1+2*sigma)*(1+sigma)))*sigma^1*6;
                A(5,3) = (1/((1+3*sigma)*(1+2*sigma)*(1+sigma)))*sigma^2*11;
                A(5,2) = (1/((1+3*sigma)*(1+2*sigma)*(1+sigma)))*sigma^3*6;

                A(6,6) = (1/((1+4*sigma)*(1+3*sigma)*(1+2*sigma)*(1+sigma)));
                A(6,5) = (1/((1+4*sigma)*(1+3*sigma)*(1+2*sigma)*(1+sigma)))*sigma^1*10;
                A(6,4) = (1/((1+4*sigma)*(1+3*sigma)*(1+2*sigma)*(1+sigma)))*sigma^2*35;
                A(6,3) = (1/((1+4*sigma)*(1+3*sigma)*(1+2*sigma)*(1+sigma)))*sigma^3*50;
                A(6,2) = (1/((1+4*sigma)*(1+3*sigma)*(1+2*sigma)*(1+sigma)))*sigma^4*24;
            end
            
            if(points > 3)
                A(7,7) = (1/((1+5*sigma)*(1+4*sigma)*(1+3*sigma)*(1+2*sigma)*(1+sigma)));
                A(7,6) = (1/((1+5*sigma)*(1+4*sigma)*(1+3*sigma)*(1+2*sigma)*(1+sigma)))*sigma^1*15;
                A(7,5) = (1/((1+5*sigma)*(1+4*sigma)*(1+3*sigma)*(1+2*sigma)*(1+sigma)))*sigma^2*85;
                A(7,4) = (1/((1+5*sigma)*(1+4*sigma)*(1+3*sigma)*(1+2*sigma)*(1+sigma)))*sigma^3*225;
                A(7,3) = (1/((1+5*sigma)*(1+4*sigma)*(1+3*sigma)*(1+2*sigma)*(1+sigma)))*sigma^4*274;
                A(7,2) = (1/((1+5*sigma)*(1+4*sigma)*(1+3*sigma)*(1+2*sigma)*(1+sigma)))*sigma^5*120;

                A(8,8) = (1/((1+6*sigma)*(1+5*sigma)*(1+4*sigma)*(1+3*sigma)*(1+2*sigma)*(1+sigma)));
                A(8,7) = (1/((1+6*sigma)*(1+5*sigma)*(1+4*sigma)*(1+3*sigma)*(1+2*sigma)*(1+sigma)))*sigma^1*21;
                A(8,6) = (1/((1+6*sigma)*(1+5*sigma)*(1+4*sigma)*(1+3*sigma)*(1+2*sigma)*(1+sigma)))*sigma^2*175;
                A(8,5) = (1/((1+6*sigma)*(1+5*sigma)*(1+4*sigma)*(1+3*sigma)*(1+2*sigma)*(1+sigma)))*sigma^3*735;
                A(8,4) = (1/((1+6*sigma)*(1+5*sigma)*(1+4*sigma)*(1+3*sigma)*(1+2*sigma)*(1+sigma)))*sigma^4*1624;
                A(8,3) = (1/((1+6*sigma)*(1+5*sigma)*(1+4*sigma)*(1+3*sigma)*(1+2*sigma)*(1+sigma)))*sigma^5*1764;
                A(8,2) = (1/((1+6*sigma)*(1+5*sigma)*(1+4*sigma)*(1+3*sigma)*(1+2*sigma)*(1+sigma)))*sigma^6*720;
            end
        end
    end

    % Find the star moment set (k=0 to 2N-1) by inversion of the system of 
    % equations suggested by Eq. S38
    mstars = A\m_2Nminus;
      
    % Approximate the quadrature points associated with the star moment set
    % using Product Difference
    [w,x,werror] = ALL_productDifference(0,points,mstars(1:2*points));

    % Define error flag (we will throw out this guess for sigma_b if 
    % use_prev_flag == 1)
    use_prev_flag = 0;
        
    % Check the realizability (validity) of the star moment set by
    % calculating the Hankel determinants
    [hankel_lo, hankel_upp] = BEQMOM_hankelDet(points, mstars);
    test = hankel_lo(hankel_lo < 0);
    test2 = hankel_upp(hankel_upp < 0);
        
    % If werror = 1, the Product Difference algorithm failed
    % If test or test2 is not empty (any Hankel determinant is negative)
    % the star moment set is not realizable
    if((~isempty(test) || ~isempty(test2)) || werror == 1)
        %disp('Tested value led non-realizable moments.');
        use_prev_flag = 1;
    end
    
    % Estimate the 2Nth star moment (the guess for the 2Nth moment given 
    % the current guess for sigma_b) using the quadrature approximation 
    % (Eq. S12)
    mstar_2N = sum(w.*(x.^(2*points)));

    % Calculate J using Eq. S39
    if(points == 1)
        J = m_2n - (1/(1+sigma))*( mstar_2N + sigma*mstars(2) );
    elseif(points == 2)
        J = m_2n - (1/((1+3*sigma)*(1+2*sigma)*(1+sigma)))*( mstar_2N + sigma*6*mstars(4) + sigma^2*11*mstars(3) + sigma^3*6*mstars(2) );
    elseif(points == 3)
        J = m_2n - (1/((1+5*sigma)*(1+4*sigma)*(1+3*sigma)*(1+2*sigma)*(1+sigma)))*( mstar_2N + sigma*15*mstars(6) + sigma^2*85*mstars(5) + sigma^3*225*mstars(4) + sigma^4*274*mstars(3) + sigma^5*120*mstars(2));
    elseif(points == 4)
        J = m_2n - (1/((1+7*sigma)*(1+6*sigma)*(1+5*sigma)*(1+4*sigma)*(1+3*sigma)*(1+2*sigma)*(1+sigma)))*( mstar_2N + sigma*28*mstars(8) + sigma^2*322*mstars(7) + sigma^3*1960*mstars(6) + sigma^4*6769*mstars(5) + sigma^5*13132*mstars(4) + sigma^6*13068*mstars(3) + sigma^7*5040*mstars(2));
    else
        disp('Number of quadrature points must be between 1 and 4.');
    end
        
end

