function [ hankel_lo, hankel_upp ] = BEQMOM_hankelDet(points, moms)

% BEQMOM_HANKELDET calculates the Hankel determinants according to Dale et 
% al. (2016) Eq. S40 in order to test whether the moment set generated by 
% the model for the given time step is realizable (valid)

%   INPUTS: 
%   points         - Number of primary quadrature points 
%   moms           - Moment set to test for realizability
%
%   OUTPUTS:
%   hankel_lo      - Vector of lower Hankel determinants
%   hankel_upp     - Vector of upper Hankel determinants

    % Normalize the moment set by the zeroth moment
    moms(:,1) = moms(:,1)./moms(1,1);

    hankel_lo = zeros(2*points,1);
    hankel_upp = zeros(2*points,1);

    for m=0:(points-1)
        m_ind = m + 1;

        Hlow_2m=zeros(m+1,m+1);
        Hupp_2mplus=zeros(m+1,m+1); 
        Hlow_2mplus=zeros(m+1,m+1);  
        Hupp_2m=zeros(m,m);

        
        % Hlow_2m
        for i = 0:m
            for j = 0:m
                i_ind = i+1;
                j_ind = j+1;

                subscpt = i+j;

                if(subscpt+1 < 1)
                  Hlow_2m(i_ind,j_ind) = moms(1,1);
                else
                  Hlow_2m(i_ind,j_ind) = moms(subscpt+1,1);
                end
            end
        end
        hankel_lo(2*m_ind-1,1)=det(Hlow_2m);

        % Hupp_2mplus
        for i = 0:m
            for j = 0:m
                i_ind = i+1;
                j_ind = j+1;

                subscpt1 = i+j;
                subscpt2 = i+j+1;

                if( subscpt1+1 < 1 || subscpt2+1 < 1 )
                  Hupp_2mplus(i_ind,j_ind) = moms(1,1);
                else
                  Hupp_2mplus(i_ind,j_ind) = moms(subscpt1+1,1)-moms(subscpt2+1,1);
                end
            end
        end
        hankel_upp(2*m_ind,1)=det(Hupp_2mplus);

        % Hlow_2mplus
        for i = 0:m
            for j = 0:m
                i_ind = i+1;
                j_ind = j+1;

                subscpt1 = i+j+1;

                if(subscpt1 + 1 < 1)
                  Hlow_2mplus(i_ind,j_ind) = moms(1,1);
                else
                  Hlow_2mplus(i_ind,j_ind) = moms(subscpt1 + 1,1);
                end
            end
        end
        hankel_lo(2*m_ind,1)=det(Hlow_2mplus);

        % Hupp_2m
        for i = 0:(m-1)
            for j = 0:(m-1)
                i_ind = i+1;
                j_ind = j+1;

                subscpt1 = i+j+1;
                subscpt2 = i+j+2;

                if( subscpt1+1 < 1 || subscpt2+1 < 1 )
                  Hupp_2m(i_ind,j_ind) = moms(1,1);
                else
                  Hupp_2m(i_ind,j_ind) = moms(subscpt1+1,1)-moms(subscpt2+1,1);
                end
            end
        end
        hankel_upp(2*m_ind-1,1)=det(Hupp_2m);

        if(m==0)
            hankel_lo(2*m_ind-1,1)=moms(1,1);
            hankel_upp(2*m_ind-1,1)=moms(1,1);
        end

    end

end

