function [efflux,a,b] = DQMOM_dissFlux(points,x,w,A_i)

% DQMOM_DISSFLUX calculates the efflux of NPs during each time step due to
% complete NP dissolution.  This approach (the ratio constraint method) was 
% first presented by Fox et al. (2008).
%    
%   INPUTS:
%   points         - Number of quadrature points
%   x              - vector of weights at the given time step
%   w              - vector of abscissas at the given time step
%   A_i            - Evaluation of the rate law (Eq. S6, Dale et al. 2016)
%
%   OUTPUTS:
%   efflux         - Approximation of the efflux (loss in particle number 
%                    due to dissolution during the time step)
%   a              - Change in the weights during the timestep due to 
%                    dissolution
%   b              - Change in the weighted abscissas during the timestep
%                    due to dissolution

    % Toggle warning messages on/off (they can impede runtimes)
    warning('off','MATLAB:nearlySingularMatrix')

    % Eq. S30 (middle) in Dale et al. (2016)
    E = zeros(points);
    for i=2:points
        E(i) = w(i-1)*w(i)*(x(i-1).*A_i(i)-x(i).*A_i(i-1));
    end

    % Coefficient matrix for left hand side of Eq. S30 (top)
    A = zeros(points,points);
    A(1,:)=1;
    for i = 2:points
        for j = 1:points
            if(i == j)
                A(i,j)=-w(i-1)*x(i-1);
                A(i,j-1)=w(i)*x(i);
            end
        end
    end

    % Find b_star by inversion of Eq. S30 (top)
    b_star = A\E(:,1);

    % Eq. S31
    k = 0.5; % arbitrary (positive) power to used to calculate alpha
    alpha = (k*sum(x.^(k-1).*b_star))/((k-1)*sum((x.^k).*w));
    a = alpha.*w;
    
    b = b_star+w.*A_i; % Eq. S30 (bottom)

    efflux = -sum(a); % Eq. S32

    % Do not allow efflux to go negative
    if(efflux < 0) 
        efflux = 0;
        a(:) = 0;
        b_star(:)=0;
        b = w.*A_i;
    end

    return

end


% Fox, R. O., F. Laurent, and M. Massot (2008), Numerical simulation of 
% spray coalescence in an Eulerian framework: direct quadrature method of 
% moments and multi-fluid method, Journal of Computational Physics, 227(6), 
% 3058-3088.